; JCM-1 Monitor and BIOS ROM
; Version 0.1
; Copyright: Joe Strosnider
; Episode 5 Code Date: 1/28/2018 - 2/4/2018
;	Original Features
;
; Episode 8 Code Date: 2/17/2018
;	Added support for Native Keyboard
;	Other minor changes and improvements
;
; Purpose, Monitor: A simple monitor ROM we can use to input, view and execute code.
; Purpose, BIOS: 	Callable routines that applications can use for hardware abstraction.

; Assembler Syntax: Retro Assembler - https://enginedesigns.net/apps/

.target "65C02"	; We're running a 65C02 Processor


/* ******************************************************** */
/* Constants												*/

	/* Zero Page Locations */
		; Monitor Locations 	($00 - $3F)
			CmdBufPtr			= $00	; Means two things simultaneously.
										;   1) Points to the next open space in the command buffer.
										;   2) Is equal to the length of buffer contents.
										
			StrOutPtr			= $01	; Pointer to string location

			MonCurLo			= $03	; Low byte of monitor cursor location
			MonCurHi			= $04	; High byte of monitor cursor location
			
			ADIndex				= $05	; scratch-pad of the loop location of the Address Decoder
			
			StorCurLo			= $06	; Low byte of Store routine location
			StorCurHi			= $07	; High byte of Store routine location
			SMIndex				= $08	; byte index byte parser in Store routine
		
		; BIOS ROM Locations	($40 - $7F)
			Sound_ChipSelect	= $40	; Holds which sound chip we're talking to.
			Sound_Register		= $41	; Holds the register we're modifying.
			Sound_Value			= $42	; Holds the sound value we're sending to the register.
			CharInVector		= $43   ; Vector to Character Ingest Routine
			;CharInVector+1		= $44   ;
		
	/* Other General Locations */
			CmdBuf		= $0200
			
	/* I/O Locations */
		; ACIA 0
			ACIA0_Data			= $D000 
			ACIA0_Status 		= $D001 
			ACIA0_Command 		= $D002 
			ACIA0_Control 		= $D003
		
		; Sound Control VIA
			Sound_VIA			= $D020
			Sound_OutRegB		= Sound_VIA
			Sound_DDRB	 		= Sound_VIA+2
			Sound_DDRA	 		= Sound_VIA+3
			Sound_OutRegA		= Sound_VIA+15
			Sound_SR			= $D02A
			Sound_ACR			= $D02B
			Sound_IFR			= $D02D
			Sound_IER			= $D02E
			
		; Native Keyboard	
			KeyByte 			= $D030
			KeyStatus 			= $D031
			
/* ******************************************************** */

	
/* ******************************************************** */
/* Assembler Trick to Start ROM at $0000					*/
/* You can remove the following three lines if your ROM		*/
/* programmer allows you to compile at $E000 offset			*/
			* = $0000
			.Loop $DFFF
			.byte $ff
			.endloop
			
			* = $E000	; ROM Program Starts Here	


/* ******************************************************** */
/* 						BIOS ROM 							*/
/* ******************************************************** */
B_InitACIA0		; Set ACIA 0 for 8,N,1,115200
			LDA #%00001011			;No parity. No echo. No Interrupts
			STA ACIA0_Command
			LDA #%00010000			;115,200 Baud, 8 Data bits, 1 Stop bits
			STA ACIA0_Control
			RTS	
			
/* ******************************************************** */	
B_ACIA0_In	; Gets a byte from ACIA 0, using the polling method.
			; Returns with received byte in A
			LDA ACIA0_Status		; Get ACIA Status.
			AND #$08 				; Was RX buffer empty?
			BEQ B_ACIA0_In			; Yes. Go ask again.
			LDA ACIA0_Data			; No. Gimme the byte!
			RTS

/* ******************************************************** */			
B_InitKB	; Sets the CharInVector to the Native Keyboard
			LDA #<B_ReadKB	
			STA CharInVector
			LDA #>B_ReadKB
			STA CharInVector + 1
			STZ KeyStatus			; Clears the Native Keyboard, as it might be in an unknown state.
			RTS
														
/* ******************************************************** */
B_ReadKB	; Gets a byte from the native keyboard
			; Returns with typed byte in A
			LDA KeyStatus			; Get The Keyboard Status
			AND #$80				; Is bit 7 set, meaning we have a key in the buffer?
			BEQ	B_ReadKB			; No, go check again.
			LDA KeyByte				; Yep! Read it.
			PHA						; Push key onto the stack.
KeyClear	STZ KeyStatus			; Zero Keyboard Status so the keyboard can resume scanning.
			LDA KeyStatus			; Get status again so we can check for clear.
			BNE	KeyClear			; Wait until buffer floats down to zero, because weirdness i've yet to figure out...
									; ... a software solution for a hardware problem.
			PLA						; Pull key from stack.
			RTS			

			

/* ******************************************************** */
B_InitCharIn	;Initalize the Character Ingest Vector
				;Default = ACIA0
			LDA #<B_ACIA0_In
			STA CharInVector
			LDA #>B_ACIA0_In
			STA CharInVector + 1
			LDA ACIA0_Data			;clear ACIA0, as it might be in an unknown state.
			RTS

/* ******************************************************** */				
B_CharIn 	JMP (CharInVector)


/* ******************************************************** */	
B_PrintImm	; Outputs the null-terminated string that is stored in memory directly after the subroutine call.
			; From: http://www.6502.org/source/io/primm.htm

			PLA						; Get the low part of "return" address
			STA StrOutPtr			; Store to pointer
			PLA						; Get the high part of "return" address
			STA StrOutPtr+1			; Store to pointer
	M_PI1	LDY #1					; Accounts for pointing one short. :o)
			LDA (StrOutPtr),y		; Get the next string character
			INC StrOutPtr			; Update the pointer
			BNE M_PI2				; If not, we're pointing to next character
			INC StrOutPtr+1			; Account for page crossing
	M_PI2	ORA	#0					; Set flags according to contents of Accumulator
			BEQ M_PI3				; Don't print the final NULL
			JSR B_ACIA0_Out	    	; Print the character.
			BRA M_PI1				; Go again.
	M_PI3	INC StrOutPtr
			BNE M_PI4
			INC StrOutPtr+1			; account for page crossing
	M_PI4   JMP (StrOutPtr)			; return to byte following final NULL
	

/* ******************************************************** */
B_SoundOut	;Send sound to the selected sound chip
			;Sound_ChipSelect Holds which sound chip we're talking to.
			;Sound_Register Holds the register we're modifying.
			;Sound_Value Holds the sound value we're sending to the register.
			;Returns with registers intact.

			PHA						; Save Accumulator to Stack
			LDA Sound_Register		; Pull in the register we're talking to.
			STA Sound_OutRegA		; Push it to Sound_VIA
			LDA Sound_ChipSelect	; Which sound chip are we talking to?
			BNE Latch1				; If Snd1, go do that.
			LDA #3					; Must be Snd0
			BNE LatchIt
Latch1		LDA #12					; Must be Snd1
LatchIt		STA Sound_OutRegB		; Latch the Register
			STZ Sound_OutRegB		; Clear the Sound_VIA
			
			LDA	Sound_Value			; Pull in the value for the latched register.
			STA Sound_OutRegA		; Push it to the Sound_VIA
			LDA Sound_ChipSelect	; Which sound chip are we talking to?
			BNE Write1				; If Snd1, go do that.
			LDA #2					; Must be Snd0
			BNE WriteIt
Write1 		LDA #8					; Must be Snd1
WriteIt		STA Sound_OutRegB		; Latch the Value
			STZ Sound_OutRegB		; Clear the Sound_VIA
			PLA						; Restore Accumulator
			RTS
			
/* ******************************************************** */
B_SoundClear	; Zeroes all registers in both sound chips.
				; Returns with registers intact.
				
			PHA						; Push Accumulator to the Stack.
			PHX						; Push X to the Stack.
			LDA #$FF				; Set all pins to outputs.
			STA Sound_DDRA			; Set PA Pins
			STA Sound_DDRB			; Set PB Pins
			STZ	Sound_Value			; Holds the sound value we're sending to the register.
			LDX #0					; Start with Register 0
@Loop		STZ Sound_ChipSelect 	; Select Snd0
			STX	Sound_Register		; Talk to the sound register indexed by X
			JSR B_SoundOut			; Zero the Register indexed by X
			INC Sound_ChipSelect 	; Select Snd1
			JSR B_SoundOut			; Clear the same register on the other chip.
			INX						; Increment
			CPX #16					; Have we done all 16 registers?
			BNE @Loop				; Nope! Go again.
			PLX						; Pull X from the Stack.
			PLA						; Pull Accumulator from the Stack.
			RTS
			

/* ******************************************************** */			
B_BootChime	; Plays the startup chime.
			; Destroys A, X, Y
			
			JSR B_SoundClear		; Clear any playing sounds.
			LDY #26
			LDX #0
			
	@Loop	LDA BootChimeDB,X
			STA Sound_ChipSelect
			INX
			LDA BootChimeDB,X
			STA Sound_Value
			INX
			LDA BootChimeDB,X
			STA Sound_Register
			INX
			JSR B_SoundOut
			DEY
			BNE @Loop
			RTS

				;chip, val, reg
BootChimeDB	.byte $00, $38, $07		; Snd0, Disable noise and enable tone on all channels, Register 7
			.byte $00, $10, $08		; Snd0, Amplitude Controlled by Envelope, Reg 8
			.byte $00, $10, $09		; Snd0, Amplitude Controlled by Envelope, Reg 9
			.byte $00, $10, $0A		; Snd0, Amplitude Controlled by Envelope, Reg 10
			.byte $00, $00, $0B		; Snd0, Env Period Coarse Definition, Reg 11 - Env Period Coarse Register
			.byte $00, $1E, $0C		; Snd0, Env Period Fine Definition; Reg 12 - Env Period Coarse Register
			.byte $00, $09, $0D		; Snd0, Env Shape Definition (Downward Taper), Reg 13 - Env Shape Register
			
			.byte $01, $38, $07		; Snd1, Disable noise and enable tone on all channels, Register 7
			.byte $01, $10, $08		; Snd1, Amplitude Controlled by Envelope, Reg 8
			.byte $01, $10, $09		; Snd1, Amplitude Controlled by Envelope, Reg 9
			.byte $01, $10, $0A		; Snd1, Amplitude Controlled by Envelope, Reg 10
			.byte $01, $00, $0B		; Snd1, Env Period Coarse Definition, Reg 11 - Env Period Coarse Register
			.byte $01, $1E, $0C		; Snd1, Env Period Fine Definition; Reg 12 - Env Period Coarse Register
			.byte $01, $09, $0D		; Snd1, Env Shape Definition (Downward Taper), Reg 13 - Env Shape Register

			.byte $01, $52, $00		; Snd1, Tone Freq A, Fine tune, Reg 0
			.byte $01, $01, $01		; Snd1, Tone Freq A, Coarse tune, Reg 1
			.byte $01, $E1, $02		; Snd1, Tone Freq B, Fine tune, Reg 2
			.byte $01, $00, $03		; Snd1, Tone Freq B, Coarse tune, Reg 3
			.byte $01, $86, $04		; Snd1, Tone Freq C, Fine tune, Reg 4
			.byte $01, $00, $05		; Snd1, Tone Freq C, Coarse tune, Reg 5
			
			.byte $00, $54, $00		; Snd0, Tone Freq A, Fine tune, Reg 0
			.byte $00, $01, $01		; Snd0, Tone Freq A, Coarse tune, Reg 1
			.byte $00, $71, $02		; Snd0, Tone Freq B, Fine tune, Reg 2
			.byte $00, $00, $03		; Snd0, Tone Freq B, Coarse tune, Reg 3
			.byte $00, $43, $04		; Snd0, Tone Freq C, Fine tune, Reg 4
			.byte $00, $00, $05		; Snd0, Tone Freq C, Coarse tune, Reg 5
			
			
/* ******************************************************** */
B_Bell		; Emits a bell sound.
			; Destroys A, X, Y
			
			JSR B_SoundClear		; Clear any playing sounds.
			LDY #7
			LDX #0
			
	@Loop	LDA BellDB,X
			STA Sound_ChipSelect
			INX
			LDA BellDB,X
			STA Sound_Value
			INX
			LDA BellDB,X
			STA Sound_Register
			INX
			JSR B_SoundOut
			DEY
			BNE @Loop
			RTS
				

BellDB		.byte $00, $38, $07
			.byte $00, $10, $08
			.byte $00, $00, $0B
			.byte $00, $03, $0C
			.byte $00, $09, $0D
			.byte $00, $3E, $00
			.byte $00, $00, $01


/* ******************************************************** */		
B_ColdBoot	; Sets hardware to power on defaults, plays the Startup Chime, "ANSI Clears" screen and says hello.
			JSR B_InitACIA0			; Get ACIA0 Ready for transmission.
			JSR B_InitCharIn		; Initialize Character Ingest Vector
			JSR B_BootChime			; Sing a pretty song.
			JSR B_PrintImm			; Say hello.
			.byte $1B,"[2J",$1B,"[H","JCM-1 Computer",$00
			
			; Falls through to the monitor.
			
			
/* ******************************************************** */
/* 						Monitor ROM 						*/
/* ******************************************************** */
M_InitMonitor	; Initalize the monitor.
			STZ MonCurLo
			STZ MonCurHi
			
/* ******************************************************** */	
M_Prompt	; Show the command prompt.
			STZ CmdBufPtr			; Set the command buffer pointer to zero.
			JSR B_PrintImm			; Advance two lines.
			.byte $0D,$0A,$0D,$0A,$00	
			
			JSR M_PtMonCur			; Print Monitor Cursor
			LDA #">"				; Load up a bracket..
			JSR B_ACIA0_Out			; send the command prompt		

/* ******************************************************** */
M_Input		; Get user input. The heart of the monitor.
			JSR B_CharIn   			; Get a byte.
			CMP #$1B				; Was it Escape?
			BEQ M_Prompt			; Yup. Bomb immediately to prompt.
			CMP #$08				; Was it Backspace?
			BEQ	M_BkSpc				; Yup. Go deal with that.
			CMP #$7F				; Was it Delete?
			BEQ	M_BkSpc				; Yup. Go deal with that.

			JSR B_ACIA0_Out  		; Otherwise, spit back the char so the user knows we got it.
			LDX CmdBufPtr			; Get the Current Buffer Length. Done here to save bytes later.
			CMP #$0D				; Is it Carriage Return?
			BEQ M_ParseCmd			; Yup. We have a command in the buffer. Go parse it.

			CPX #$FF				; Buffer overrun?
			BEQ M_Prompt			; Yes. Go prompt.
			STA CmdBuf,X    		; Otherwise, add input to buffer
			INC CmdBufPtr   		; Increment the buffer pointer
			BRA M_Input				; Go get another character.

M_BkSpc		LDX CmdBufPtr			; Get the buffer pointer.
			BEQ M_Prompt  			; Buffer empty. Go prompt.
			DEC CmdBufPtr   		; Otherwise, decrement buffer pointer
			JSR B_PrintImm			; Erase the last character on the screen.
			.byte $08," ",$08,$00
			BRA M_Input				; Go get another character.	

M_ParseCmd	LDA #$0A				; Load up a line feed.
			JSR B_ACIA0_Out 		; Spit it out.
			
			
			// keyboard diagnostics - JSS
			// native keyboard increments the command buffer pointer 7 times for each character. Why?
			// LDA CmdBufPtr
			// JSR M_PtHex
			
			// LDA #$0A				; Load up a line feed.
			// JSR B_ACIA0_Out 		; Spit it out.
			
			
			
			CPX #$0					; Remember, we've already loaded this with CmdBufPtr...
			BEQ CmdListImm 			; 0 = blank input. Go list memory.
			LDX #$0					; Ready to parse. Let's start at the beginning.
			PC_Ph1	LDA CmdBuf,X		; Let's get a byte.
					CMP #$20			; Is it a space?
					BNE PC_Ph2			; Nope, continue.
					INX					; It's a space. Increment.
					CPX CmdBufPtr		; Where are we in the command buffer?
					BEQ CmdListImm 		; Reached buffer end and found nothing but spaces. Go list memory.
					BRA PC_Ph1			; Otherwise, go get another byte.
				
			PC_Ph2	AND #$DF			; Convert to uppercase.
					CMP #"G"			; [G]o?
					BEQ CmdGo  			; Yep, go execute the memory location.
					CMP #"L"			; [L]ist memory?
					BEQ	CmdList			; Yep, go list memory location
					CMP #"S"			; [S]tore Memory?
					BEQ CmdStore		; Yep, go store memory
					CMP #"I"			; Change [I]nput Vector?
					BEQ CmdIngest		; Yep, go do that.
						
			PC_Err	JSR B_PrintImm		; fell through, must be an error
					.byte "Err",$0D,$0A,$00
					JSR B_Bell
					JMP M_Prompt

			CmdIngest
					INX					; Increment X so that ...
					LDA CmdBuf,X		; ... we can see what's right after the "I"
					CMP #"0"			; Is it Zero?
					BEQ CI_Zero			; Yep.
					CMP #"1"			; Is it One?
					BEQ CI_One			; Yep.
					BRA PC_Err	 		; Invalid input. Go error.
					
			CI_Zero	JSR B_InitCharIn	; Input Device Zero: ACIA0
					JMP M_Prompt
					
			CI_One	JSR B_InitKB		; Input Device One: Native Keyboard
					JMP M_Prompt
					
			CmdList	JSR CmdRegrab		; "Regrab" the command.
					CMP #'L'			; Compare with "L"
					BEQ CmdListImm		; All we have in the buffer is "L". Just go show memory.
					JSR M_AddrDecode	; Otherwise, we have some hex to decode. Let's do that.
										; Then come back and list memory immediately
			CmdListImm
					JSR B_PrintImm			
					.byte $1B,"[1A",$00	; Put the cursor back where it was...
					JSR M_PtMonCur		; Show the Monitor Cursor
					LDY #0			
			LMLoop	LDA (MonCurLo),Y	; Grab a byte.
					JSR M_PtHex			; Print it out
					LDA #" "			; Give us a little space...
					JSR B_ACIA0_Out		; ... would ya, huh?
					INY					; Increment
					CPY #16				; 16 bytes?
					BNE LMLoop			; Nope, go again.
					LDA MonCurLo		; Load Monitor Cursor
					ADC #15				; Move it forward.
					STA MonCurLo		; Save it out.
					BCC LMExit			; Did it roll over?
					INC MonCurHi		; Yep. Then increment hi byte
			LMExit	JMP M_Prompt

			CmdGo	JSR M_AddrDecode	; Put the Go address in MonCurLo/Hi
					JMP (MonCurLo)		; Unconditional Jump.
				
			CmdStore
					JSR CmdRegrab		; "Regrab" the command.
					CMP #'S'			; Compare with "S"
					BEQ CSMExit			; All we have in the buffer is "S". Nothing to do. Go prompt.
					INX					; Increment and...
					LDA CmdBuf,X		; ... grab the next character.
					CMP #':'			; Is it a colon?
					BEQ CSMPhaseA		; Yes. Next thing might be bytes to store. Check that.
					DEX					; Go back one, because it's not a colon.
					JSR M_AddrDecode	; Must be an address. Go decode it
					LDA CmdBuf,X		; Grab char after the address.
					CMP #':'			; Again, is it a colon?
					BNE PC_Err			; No. Error in input.
				
				CSMPhaseA	INX				; Next Char
							CPX CmdBufPtr	; Have we reached buffer end?
							BEQ CSMExit		; Yep. No bytes to store.
							
							LDA MonCurLo	; We have bytes! Copy monitor location to store routine cursor...
							STA StorCurLo	; ..so that store routine
							LDA MonCurHi	; ....can do it's
							STA StorCurHi	; ......magic
				
				CSMPhaseB	STZ SMIndex		; Zero the byte index.
							STZ MonCurLo    ; We need a clean byte to deal with the math.
							
				CSMPhaseC	LDA CmdBuf,X	; Get a byte
							JSR M_IsHex		; Ask if it's hex
							BEQ	CSMPhaseG	; Not Hex. 
							
				CSMPhaseD	LDY SMIndex		; Which nibble are we on?
							BNE CSMPhaseE	; Second nibble.
							JSR M_ToNibble	; First nibble. Shift Accumulator into MonCurLo
							INX
							CPX CmdBufPtr	; CmdBuf End?
							BNE CSMPhaseF	; Not done. Continue.
							JSR CSMStore 	; We're done.
							JSR M_PutMon	; put the monitor where it needs to be.
							BRA CSMExit 	; Go prompt
							
				CSMPhaseE	JSR M_ToNibble	; Shift Accumulator into MonCurLo	
							JSR CSMStore	; Store it
							BRA CSMPhaseH	
							
				CSMPhaseF	INC SMIndex		; Advance
							BRA CSMPhaseC	; Go again.
							
				CSMPhaseG	LDY SMIndex		; Load the Byte Index
							BEQ CSMPhaseH	; It's Zero. Need to check again.
							JSR CSMStore    ; It's one. Store previous nibble to memory.
				CSMPhaseH	JSR M_PutMon	; put the monitor where it needs to be.
							INX
				CSMPhaseI	CPX CmdBufPtr	; Reached buffer end?
							BNE CSMPhaseB	; Nope. Keep going.
				CSMExit		JMP M_Prompt	; Yup. We're done.
				
				CSMStore	LDY #0
							LDA MonCurLo		; Grab the byte to store into Accumulator
							STA (StorCurLo),Y	; store at address indicated by StorCurLo
							INC StorCurLo		; Increment StorCurLo
							BNE CSMStore2		; No rollover? Then done.
							INC StorCurHi		; Rolled over. Increment StorCurHi
				CSMStore2	RTS

			
CmdRegrab	;shortcut to duplicated code. Saves bytes.
			LDY CmdBufPtr			; Grab the pointer.
			DEY						; Decrement so we're looking at the last actual byte in the buffer.
			LDA CmdBuf,Y			; Grab the byte.
			AND #$DF				; Convert to uppercase.
			RTS
			
M_PutMon	;shortcut to duplicated code. Saves bytes.
			LDA StorCurLo			; put the monitor where it needs to be.
			STA MonCurLo			;
			LDA StorCurHi			;
			STA MonCurHi			;
			RTS
					
/* ******************************************************** */
M_AddrDecode	; Decodes the address at the current buffer pointer and puts it in MonCurLo/Hi
				; Inspired by Woz, heavily modified.
				
			STZ MonCurLo			; Zero out MonCur-- so nibble shift works properly.
			STZ MonCurHi			;	"
			STZ	ADIndex 			; Keeps track of times we've looped, since both index registers are taken up.
			INX						; because we're still pointing at the command char, not the value. :o)
	@Loop	JSR M_IsHex 			; Is it hex?
			BEQ AD_Exit 			; Not hex. Bomb.
									
			JSR M_ToNibble			; Is Hex. Shift it into Zero Page.
			INX             		; Advance text index.
			CPX CmdBufPtr 			; Compare X with CmdBufPtr.
			BEQ AD_Exit				; Equal to buffer end. We're done.
							
									; Less than buffer end? Go check to see if we're hit the 4 char limit.
			INC ADIndex				; Advance Address Decoder index.
			LDA ADIndex				; Pop it into the accumulator
			CMP #$04				; Compare it with 4.
			BNE	@Loop				; if A < 4, we're definitely not done.
	AD_Exit	RTS 					; We're done. Decoded address is in MonCurLo and MonCurHi

/* ******************************************************** */	
M_IsHex
			; Checks that a character in Accumulator is hex.
			; Valid returns original value in Accumulator
			; Invalid returns Accumulator = binary zero.
			
			; determine if it's a number nibble
			LDA CmdBuf,X			
			CMP #$30 				; compare...
			BCC NotHex				; if < "0", can't be number or letter.
			CMP #$3A 				; compare
			BCS IsAlpha				; if > "9", go check it for letter (or >= 9+1)
			RTS						; yes number. Return with value in A. If we return with something that isn't binary zero, then it's valid

	IsAlpha	;determine if it's a letter nibble.
			AND #$DF				; Convert to uppercase.
			CMP #$41 				; if < "A", skip
			BCC NotHex
			CMP #$47 				; if > "F", skip (or >= F+1)
			BCS NotHex
			ADC #$B9				; A little binary math to make our letter suitable for bit shifting
			RTS 					; Return with value in A. If we return with something that isn't binary zero, then it's valid

	NotHex	LDA #0					; If we return with EXACTLY binary zero, it's Not Hex!
			RTS

		
/* ******************************************************** */		
M_ToNibble	;Move the nibble into position. Inspired by Woz Apple 1 Monitor.
			ASL						; Move digit to MSD of A
			ASL						; "
			ASL						; "
			ASL						; "
			LDY #4  				; Have to shift four times.
	@Loop	ASL						; Digit left, MSB to carry
			ROL MonCurLo			; Rotate into LSD
			ROL MonCurHi			; Rotate into MSD
			DEY						; Done 4 shifts?
			BNE	@Loop				; No, loop
			RTS
	
/* ******************************************************** */
M_PtMonCur	; Prints the monitor cursor location on screen.
			LDA MonCurHi			; Load MSB
			JSR M_PtHex				; Convert and send MSB
			LDA MonCurLo			; Load LSB
			JSR M_PtHex				; Convert and send LSB
			LDA #":"				; Load up your colon...
			JSR B_ACIA0_Out			; ... and empty it it out.
			RTS						; ...... Bathroom humor!

/* ******************************************************** */
M_PtHex	; Prints Accumulator directly to screen as a hex byte.
		; Author: the one and only Woz.
		; Returns with LSD in Accumulator
					
			PHA						;Save A for LSD
			LSR						; /========================-
			LSR						;/| Shift MSD ...         /						<= Look at that
			LSR						;\| ... to LSD position   \						     ASCII Art! :p
			LSR						; \========================-
			JSR	PtHex				;Output hex digit, some fall through magic...
			PLA						;Restore A
PtHex		AND	#%00001111			;Mask LSD for hex print
			ORA	#"0"				;Add "0"
			CMP	#"9"+1				;Is it a decimal digit?
			BCC	B_ACIA0_Out			;Yes! output it, with a sneaksy falltrhough.
			ADC	#6					;Add offset for letter A-F
		; Falls through. Sneaksy. :o) Will require later patch when Output Vector is Changed
		
/* ******************************************************** */
B_ACIA0_Out		; Sends a byte out ACIA 0
				; Byte to send must be in A
				; Returns with registers intact	

			; Original version that uses transmit buffer status bit.
			; Incompatible with most WDC 65C51 chips due to bug.
			/*				
						PHA					; Put A on the stack
			A0SB_Loop	LDA ACIA0_Status	; Get ACIA Status.
						AND #$10 			; Is TX buffer full?
						BEQ A0SB_Loop		; Yes. Go ask again.
						PLA					; No. Restore A
						STA ACIA0_Data		; And send the byte out.
						RTS
			*/

			; New version that simply waits dependant on clock speed.
			; Allows us to use the WDC 65C51, and tune the wait depending on clock speed while avoiding the TX buffer bit stuck bug.
			; Serial Clock is 115200
			; 8N1 = 12,800 bytes/second or 0.000078125 seconds/byte
			; 1Mhz Clock = 0.000001 seconds/cycle
			; One byte takes 0.000078125/0.000001 Cycles or..
			; 	... 78.125 cycles
			; So, @ 1Mhz, we have to wait a minimum 79 cycles between sending bytes.

									; JSR here took 6 Cycles
			PHX						; 3 cycles
			LDX #16					; 2 Cycles
			
			; 11 cycles by the time we get here. We have 68 cycles to burn.
			
	@Loop	DEX						; Each pass through the loop is 4 cycles, with the last taking five.
			BNE @Loop				; So, this loop takes up 69
			STA ACIA0_Data			; By the time we're here, we've eaten exactly 80 cycles. Send the byte out. 
			PLX
			RTS
			
/* ******************************************************** */
			* = $FFFA	; Vectors
			.word B_ColdBoot		;$FFFA/$FFFB - NMI
			.word B_ColdBoot		;$FFFC/$FFFD - RESET
			.word B_ColdBoot		;$FFFE/$FFFF - IRQ/BRK
